/* global browser, app, Port, _, cfg:true */
"use strict";

var init = function(win, doc, head) {
	// cfg is not available in normal web-pages, but in extension pages only
	if (typeof Port === "undefined" || typeof cfg === "object" || !doc || !doc.body) {
		return;
	}

	var img = doc.images[0];
	var transform = {
		flip: function(el, ori) {
			if (!el.scale) {
				el.scale = {"h": 1, "v": 1};
			}

			el.scale[ori ? "h" : "v"] *= -1;
			ori = el.scale.h !== 1 || el.scale.v !== 1 ?
				"scale(" + el.scale.h + "," + el.scale.v + ")" :
				"";

			if (el.curdeg) {
				ori += " rotate(" + el.curdeg + "deg)";
			}

			el.style[browser["transform"]] = ori;
		}
	},
	pdsp = function(e, d, p) {
		if (!d) {
			e.preventDefault();
		}

		if (p) {
			return;
		}

		if (e.stopImmediatePropagation) {
			e.stopImmediatePropagation();
		}
		else {
			e.stopPropagation();
		}
	},
	shortcut = {
		specKeys: {
			8: "Backspace", 9: "Tab", 13: "Enter",
			16: "shift", 17: "ctrl", 18: "alt", // control keys are not capital
			27: "Esc", 32: "Space",
			33: "PgUp", 34: "PgDn", 35: "End", 36: "Home",
			37: "Left", 38: "Up", 39: "Right", 40: "Down",
			45: "Ins", 46: "Del",
			96: "0", 97: "1", 98: "2", 99: "3", 100: "4",
			101: "5", 102: "6", 103: "7", 104: "8", 105: "9",
			106: "*", 107: "+", 109: "-", 110: ".", 111: "/",
			186: ";", 187: "=", 188: ",", 189: "-", 190: ".",
			191: "/", 192: "`", 219: "[", 220: "\\", 221: "]", 222: "'",
			112: "F1", 113: "F2", 114: "F3", 115: "F4", 116: "F5", 117: "F6",
			118: "F7", 119: "F8", 120: "F9", 121: "F10", 122: "F11", 123: "F12"
		},
		isModifier: function(e) {
			return e.which > 15 && e.which < 19;
		},
		key: function(e) {
			return this.specKeys[e.which] || String.fromCharCode(e.which).toUpperCase();
		}
	};

	if (browser.opera || browser.firefox) {
		img.style.display = "none";
	}

	head = head && (head.data || head.message || head); // Opera / Maxthon / Chrome, Safari, Firefox

	if (browser.opera) {
		win.donotrun = img.error = true;
		img.naturalWidth = 0;

		img.onclick = win.ondragstart = win.onkeypress = win.onmousedown =
									win.onmouseup = win.onmousemove = null;
	}

	["class", "style"].forEach(function(attr) {
		doc.documentElement.removeAttribute(attr);
		doc.body.removeAttribute(attr);
	});

	cfg = JSON.parse(head.prefs);

	var root, init_params, sub_init,
		after_calc,
		moving,
		winW, winH,
		sX, sY,
		URL = img.src,
		noFit = {cur: false, real: false},
		lastEvent = {},
		progress = null,
		cancelAction = false,
		fZoom = null,
		dragSlide = [],
		borderSize = 0,
		MAXSIZE = 0x7fff;

	if (head = doc.querySelector("head")) {
		doc.documentElement.removeChild(head);
	}

	head = doc.createElement("head");

	if (cfg.favicon) {
		root = doc.createElement("link");
		root.rel = "shortcut icon";
		root.href = cfg.favicon === "%url"
			? win.location.href
			: cfg.favicon.replace("%url", encodeURIComponent(win.location.href));
	}

	if (root) {
		head.appendChild(root);
	}

	root = doc.createElement("style");
	root.appendChild(doc.createTextNode(
		"html, body{width: 100%; height: 100%; -webkit-user-select: none; -moz-user-select: none; -ms-user-select: none}" +
		"body {cursor: default; margin:0; padding: 0; font: 12px 'Trebuchet MS', sans-serif}" +
		"h2 {margin: 0; padding-top: 15%; text-align: center; text-shadow: 1px 1px 15px #000; color: #fff; font: 700 30px 'Trebuchet MS', sans-serif}" +
		".m_1 {max-width: 100%; max-height: 100%; width: auto; height: auto}" +
		".m_2 {width: 100%; height:auto} .m_3 {height: 100%; width: auto}" +
		"a {text-decoration: none}" +
		"#menu {position: fixed; " + browser["transition_css"] + ": left .3s; width: 50px; height: 33.33%; top: 0}" +
		"ul {display: inline-block; text-align: center; margin: 0; padding: 5px 10px; list-style: none; font-size: 25px; font-weight: 700; background: rgba(255, 255, 255, 0.9)}" +
		"li {display: inline-block; position: relative}" +
		"li[data-cmd] {cursor: pointer}" +
		"li[data-cmd=cycle], li[data-cmd=rotate] {position: relative}" +
		"li[data-cmd=cycle]::after {content: '\u2195' ; position: absolute; left: 25%}" +
		"li[data-cmd=rotate]::after {content: '\u21BB'; position: absolute; left: 0%}" +
		"li ul {position: absolute; left: 100%; top: 0; visibility: hidden; opacity: 0; " + browser["transition_css"] + ": visibility .4s, opacity .2s .3s; text-align: left}" +
		"li:hover ul {display: block; opacity: 1; visibility: visible}" +
		"li ul>li {display: block !important; font-size: 15px; padding: 3px 0; color: black}" +
		".send_hosts li > a {background-size: 16px 16px; background-repeat: no-repeat; background-position: 0 3px; padding-left: 25px;}" +
		"#menu>ul>li {margin: 0 10px}" +
		"#menu>ul>li:hover {color: silver}" +
		"li ul>li {white-space: nowrap}" +
		".filters>ul {font-size: 15px}" +
		".filters>form {margin: 0; padding: 0}"));

	head.appendChild(root);
	doc.documentElement.insertBefore(head, doc.body);

	sub_init = function() {
		if (!cfg.imgInfo) {
			cfg.imgInfo = false;
		}

		var style = doc.getElementById(app.name + "_CSS");

		if (!style) {
			style = doc.createElement("style");
			style.id = app.name + "_CSS";
			head.appendChild(style);
		}

		if (!img || !img.width) {
			return;
		}

		style.textContent = "body>img{background-clip: padding-box;" +
		browser["box-sizing_css"] + ": border-box; position: absolute;" +
		"max-width:" +
			Math.min(
					MAXSIZE,
					Math.floor(img.naturalWidth * MAXSIZE / img.naturalHeight)
				) +
			"px;max-height:" +
			Math.min(
					MAXSIZE,
					Math.floor(img.naturalHeight * MAXSIZE / img.naturalWidth)
				) + "px}" +
			(cfg.customCSS || "html{overflow: hidden; background-color: #f3f3f5}");

		cfg.hiddenScrollbars = win.getComputedStyle(root).overflow === "hidden"
										|| win.getComputedStyle(doc.body).overflow === "hidden";

		var menu = doc.body.appendChild(doc.createElement("div"));
		menu.id = "menu";

		if (win.getComputedStyle(menu).display === "none") {
			return;
		}

		menu.innerHTML = '<ul>' +
				'<li data-cmd="cycle">&#8596;</li>' +
				'<li data-cmd="flip">&#8651;</li>' +
				'<li data-cmd="rotate">&#8634;</li>' +
				'<li class="send_hosts">&#128269;' +
					'<ul>' +
						'<li><a href="http://www.tineye.com/search?sort=size&order=desc&url=%">TinEye</a></li>' +
						'<li><a href="https://images.google.com/searchbyimage?image_url=%">Google</a></li>' +
						'<li><a href="http://images.yandex.ru/yandsearch?rpt=imagecbir&img_url=%">Yandex</a></li>' +
						'<li><a href="http://karmadecay.com/%">KarmaDecay</a></li>' +
						'<li><a href="http://zxing.org/w/decode?full=true&u=%">ZXing QRCode</a></li>' +
						'<li><a href="http://imgops.com/%">ImgOps</a></li>' +
						'<li><a href="http://pixlr.com/editor/?image=%">Pixlr</a></li>' +
					'</ul>' +
				'</li>' +
				'<li class="filters">&#8801;' +
					'<form><ul>' +
						'<li><input type="range" min="0" max="250" step="20" value="100" unit="%"> brightness</li>' +
						'<li><input type="range" min="0" max="300" step="25" value="100" unit="%"> contrast</li>' +
						'<li><input type="range" min="0" max="1000" step="50" value="100" unit="%"> saturate</li>' +
						'<li><input type="range" min="0" max="100" step="25" value="0" unit="%"> grayscale</li>' +
						'<li><input type="range" min="0" max="100" step="100" value="0" unit="%"> invert</li>' +
						'<li><input type="range" min="0" max="100" step="20" value="0" unit="%"> sepia</li>' +
						'<li><input type="range" min="0" max="360" step="36" value="0" unit="deg"> hue-rotate</li>' +
						'<li><input type="range" min="0" max="20" step="1" value="0" unit="px"> blur</li>' +
					'</ul></form>' +
				'</li>' +
			'</ul>';

		menu.addEventListener("mousedown", function(e) {
			var t = e.target;

			if (t.href && t.href.slice(-1) === "%") {
				t.href = t.href.slice(0, -1) + encodeURIComponent(win.location.href);
			}

			if (t.textContent) {
				pdsp(e);
			}
		}, false);

		menu.style.cssText = "-webkit-filter: blur(1px); filter: blur(1px)";

		if (menu.style.webkitFilter || menu.style.filter) {
			img.filters = {};
			menu.onchange = function(e) {
				var t = e.target, f = "";

				if (t.value) {
					if (t.value === t.defaultValue) {
						delete img.filters[e.target.parentNode.textContent];
					}
					else {
						img.filters[t.parentNode.textContent.trim()] = t.value + t.getAttribute("unit");
					}

					for (var fname in img.filters) {
						f += fname + "(" + img.filters[fname] + ") ";
					}
				}

				img.style.webkitFilter = f;
				img.style.filter = f;
			};
		}
		else {
			menu.lastElementChild.lastElementChild.style.display = "none";
		}

		menu.style.cssText = "display: none; left: -" + menu.offsetWidth + "px";

		// load favicons only when the menu item is hovered the first time
		menu.querySelector(".send_hosts").onmouseover = function() {
			this.onmouseover = null;

			[].forEach.call(this.querySelectorAll(".send_hosts>ul>li>a"), function(el) {
				el.style.backgroundImage = "url(http://" + (el.getAttribute("data-favicon") || (el.host + "/favicon.ico")) + ")";
			});
		};

		var handle_cmd = function(cmd, e) {
			if (e.button === 1) {
				return;
			}

			var p = (-e.wheelDelta || e.deltaY || e.detail) > 0 || e.button === 3 ? 1 : 0;

			if (cmd === "cycle") {
				img.cycle(p);
			}
			else if (cmd === "flip") {
				transform.flip(img, p);
			}
			else if (cmd === "rotate") {
				img.rotate(p, e.shiftKey);
			}
			else {
				p = null;
			}

			if (p !== null) {
				pdsp(e);
			}
		};

		menu.addEventListener(browser["wheel"], function(e) {
			var t = e.target;

			if (t.type === "range") {
				t.value = Math.max(t.getAttribute("min"), Math.min(parseInt(t.value, 10) + t.getAttribute("step") * ((-e.wheelDelta || e.deltaY || e.detail) > 0 ? -1 : 1), t.getAttribute("max")));
				menu.onchange(e);
				pdsp(e);
			}
			else if (t.getAttribute("data-cmd") !== "flip") {
				handle_cmd(t.getAttribute("data-cmd"), e);
			}
		}, false);

		menu.onclick = function(e) {
			if (e.target.hasAttribute("data-cmd")) {
				handle_cmd(e.target.getAttribute("data-cmd"), e);
			}
		};

		menu.oncontextmenu = function(e) {
			var t = e.target;

			if (t.type === "range") {
				t.value = t.defaultValue;
				menu.onchange(e);
				pdsp(e);
			}
			else if (t.hasAttribute("data-cmd")) {
				handle_cmd(t.getAttribute("data-cmd"), e);
			}
		};

		menu.addEventListener(browser["transitionend"], function() {
			if (this.style.left[0] === "-") {
				menu.style.display = "none";
			}
		}, false);

		if (win.Node.prototype && !win.Node.prototype.contains) {
			win.Node.prototype.contains = function(n) {
				return n instanceof Node && (this === n || !!(this.compareDocumentPosition(n) & 16));
			};
		}

		menu.onmouseover = function() {
			if (menu.mtimer) {
				clearTimeout(menu.mtimer);
				menu.mtimer = null;
				menu.style.left = "0";
			}
		};

		menu.onmouseout = function(e) {
			if (!this.contains(e.relatedTarget)) {
				doc.addEventListener("mousemove", mmove, false);
				menu.mtimer = setTimeout(function() {
					menu.style.left = "-" + menu.offsetHeight + "px";
					menu.mtimer = null;
				}, 1000);
			}
		};

		function mmove(e) {
			if (!moving && e.clientX < 40 && e.clientY < win.innerHeight / 3) {
				if (menu.style.display !== "block") {
					menu.style.display = "block";

					setTimeout(function() {
						menu.style.left = "0";
					}, 50);

					doc.removeEventListener("mousemove", mmove, false);
				}
			}
		}

		doc.addEventListener("mousemove", mmove, false);
	};

	init = function() {
		if (!(img.width || init_params === null)) {
			if (init_params && ++init_params.loop >= init_params.maxLoop && progress) {
				clearInterval(progress);
			}

			return;
		}

		if (progress) {
			clearInterval(progress);
		}

		if (win.location.href !== img.src) {
			try {
				win.history.pushState(null, null, img.src);
			} catch (ex) {}
		}

		if (browser.opera || browser.firefox || !doc.images.length) {
			doc.body.textContent = "";
			doc.body.appendChild(img);
		}
		else {
			doc.body.replaceChild(img, doc.images[0]);
		}

		sub_init();

		after_calc = function() {
			if (img.calcFit) {
				winH = doc.compatMode[0] === "B" ? doc.body : root;
				winW = winH.clientWidth;
				winH = winH.clientHeight;

				img.setPos();

				if (!cfg.wheelZoom) {
					doc[(img.width > winW || img.height > winH ? "add" : "remove") + "EventListener"](
						browser["wheel"],
						onWheel,
						false
					);
				}

				setTimeout(function() {
					// Gecko and WebKit/Blink render scrollbars for a split of a second
					// even if the image fits into the view-port
					winH = doc.compatMode[0] === "B" ? doc.body : root;
					winW = winH.clientWidth;
					winH = winH.clientHeight;

					img.calcFit();
					img.setInfo();
					img.setCur();
				}, 0xf);
			}
		};

		img.resize = function(m, w) {
			if (m === -1 || (m === -1 && fZoom)) {
				if (w) {
					this.style.width = w;
				}
			}

			if (m === void 0) {
				m = 1;
			}

			if (m !== -1 && this.mode === -1) {
				this.style.width = this.style.height = "";
			}

			this.mode = m;
			this.className = "m_" + m;

			if (this.naturalWidth) {
				after_calc();
			}
		};

		img.setInfo = function() {
			if (!cfg.imgInfo || !(img.naturalWidth && img.naturalHeight)) {
				return;
			}

			doc.title = cfg.imgInfo.replace(/%(o?[wh]|url|name|ratio|perc)/g, function(a, a1) {
					switch (a1) {
						case "w": return img.width;
						case "h": return img.height;
						case "ow": return img.naturalWidth;
						case "oh": return img.naturalHeight;
						case "url": return win.location;
						case "name": return img.alt;
						case "ratio":
							return Math.round(img.width / img.height * 100) / 100;
						case "perc":
							return ~~(0.5 + 1e4 * (img.width / img.naturalWidth)) / 100;
					}
				});
		};

		img.setPos = function() {
			var s = this.style;

			if (cfg.center) {
				s.top = Math.max(0, (winH - this.offsetHeight) / 2) + "px";
				s.left = Math.max(0, (winW - this.offsetWidth) / 2) + "px";
			}
			else {
				s.top = s.left = "0";
			}

			this.box = this.getBoundingClientRect();

			if (this.box.left < 0) {
				s.left = (parseInt(s.left, 10) - this.box.left - win.pageXOffset) + "px";
			}

			if (this.box.top < 0) {
				s.top = (parseInt(s.top, 10) - this.box.top - win.pageYOffset) + "px";
			}
		};

		img.calcFit = function() {
			var box = this.box || this.getBoundingClientRect();

			noFit = {
				cur: (box.width <= winW && box.height <= winH),
				real: (this.naturalWidth <= winW && this.naturalHeight <= winH)
			};
		};

		img.wheelZoom = function(e) {
			pdsp(e);
			stopScroll();

			var w = img.offsetWidth, h = img.offsetHeight;

			if ((-e.wheelDelta || e.deltaY || e.detail) > 0) {
				img.resize(-1, Math.max(1, w * 0.75) + "px");
			}
			else {
				var width = w * (4 / 3);

				img.resize(-1, (width > 10 ? width : width + 3) + "px");
			}

			if (e.keypress || e.target.nodeName.toUpperCase() === "IMG") {
				win.scrollTo(
					(e.offsetX || e.layerX || 0) * img.offsetWidth / w - e.clientX + borderSize,
					(e.offsetY || e.layerY || 0) * img.offsetHeight / h - e.clientY + borderSize
				);
			}
		};

		if (cfg.wheelZoom) {
			doc.addEventListener(browser["wheel"], img.wheelZoom, false);
		}
		else {
			var onWheel = function(e) {
				var x = 0, y = ((-e.wheelDelta || e.deltaY || e.detail) > 0 ? winH : -winH) / 5;

				if (img.width <= winW && img.height > winH) {
					if (!cfg.hiddenScrollbars) {
						return;
					}
				}
				else if (img.height <= winH && img.width > winW
					|| e.clientX < winW / 3 && e.clientY < winH / 3) {
					x = (y < 0 ? -winW : winW) / 5;
					y = 0;
				}
				else if (!cfg.hiddenScrollbars) {
					return;
				}

				win.scrollBy(x, y);
				e.preventDefault();
			};
		}

		img.setCur = function() {
			if (this.width > winW || this.height > winH) {
				this.style.cursor = "move";
			}
			else if (
				((winH >= this.height && winW >= this.width)
					&& (winH < this.naturalHeight || winW < this.naturalWidth))
				|| (noFit.cur && noFit.real
					&& (this.height < this.naturalHeight || this.width < this.naturalWidth))) {
				this.style.cursor = browser["zoom-in"];
			}
			else {
				this.style.cursor = "default";
			}
		};

		win.addEventListener("resize", after_calc, false);

		doc.addEventListener("contextmenu", function(e) {
			doc.removeEventListener("mousemove", onmove, false);

			if (progress) {
				clearTimeout(progress);
				progress = null;
			}

			if (cancelAction) {
				e.preventDefault();
			}
		}, false);

		var x, y;

		var onmove = function(e) {
			x = e.clientX;
			y = e.clientY;

			if (x === lastEvent.clientX && y === lastEvent.clientY) {
				return;
			}
			else if (progress) {
				clearTimeout(progress);
				progress = lastEvent.button = null;
			}

			if (sX === true || noFit.cur) {
				doc.removeEventListener("mousemove", onmove, false);
				return;
			}

			if (!moving) {
				moving = win.requestAnimationFrame(onmoveonmoveframe);
			}

			if (dragSlide.length !== 3) {
				dragSlide = [[x, y], [x, y], [x, y]];
				return;
			}

			// Opera fires move event before mouseup
			if (dragSlide[2][0] === x && dragSlide[2][1] === y) {
				return;
			}

			dragSlide[0] = [dragSlide[1][0], dragSlide[1][1]];
			dragSlide[1] = [dragSlide[2][0], dragSlide[2][1]];
			dragSlide[2] = [x, y];

			// e.timeStamp;
			img.dragSlideTime = Date.now();
		};

		var pan = function() {
			win.scrollBy(sX - x, sY - y);

			sX = x;
			sY = y;
		};

		var onmoveonmoveframe = function() {
			pan();
			moving = null;
		};

		var stopScroll = function(e) {
			if (dragSlide.length) {
				clearInterval(progress);
				progress = null;
				cancelAction = false;
				dragSlide.length = 0;
				img.dragSlideTime = false;
			}

			if (e) {
				doc.removeEventListener(browser["wheel"], stopScroll, false);
			}
		};

		var startScroll = function() {
			progress = setInterval(function() {
				win.scrollBy(dragSlide[0], dragSlide[1]);

				if (dragSlide[0]) {
					if (Math.abs(dragSlide[0]) < 1) {
						dragSlide[0] = 0;
					}

					dragSlide[0] /= 1.11;
				}

				if (dragSlide[1]) {
					if (Math.abs(dragSlide[1]) < 1) {
						dragSlide[1] = 0;
					}

					dragSlide[1] /= 1.11;
				}

				var
					atRight = root.scrollWidth - win.pageXOffset === winW,
					atBottom = root.scrollHeight - win.pageYOffset === winH;

				if (!(dragSlide[0] && dragSlide[1])
						|| (!win.pageYOffset && !win.pageXOffset) || (!win.pageYOffset && atRight)
						|| (atBottom && !win.pageXOffset) || (atBottom && atRight)) {
					stopScroll();
				}
			}, 25);
		};

		var draw_mask = function(e) {
			if (++fZoom.counter % 3) {
				return;
			}

			var
				x = e.clientX - fZoom.left,
				y = e.clientY - fZoom.top;

			if (e.ctrlKey) {
				var
					rx = fZoom.prevX ? x - fZoom.prevX : 0,
					ry = fZoom.prevY ? y - fZoom.prevY : 0;

				fZoom.prevX = x;
				fZoom.prevY = y;

				fZoom.x += rx;
				fZoom.y += ry;
				fZoom.X += rx;
				fZoom.Y += ry;
			}
			else {
				if (fZoom.prevX) {
					delete fZoom.prevX;
					delete fZoom.prevY;
				}

				fZoom.w = Math.abs(fZoom.X - x);
				fZoom.h = Math.abs(fZoom.Y - y);
				fZoom.x = fZoom.X < x ? fZoom.X : fZoom.X - fZoom.w;
				fZoom.y = fZoom.Y < y ? fZoom.Y : fZoom.Y - fZoom.h;
			}

			img.mctx.clearRect(0, 0, img.mask.width, img.mask.height);
			img.mctx.fillRect(0, 0, img.mask.width, img.mask.height);
			img.mctx.clearRect(fZoom.x, fZoom.y, fZoom.w, fZoom.h);
		};

		img.addEventListener("mousedown", function(e) {
			if (e.button === 1 || e.ctrlKey || e.altKey) {
				return;
			}

			if ((e.button === 0 && img.mode < 4) || e.button === 2) {
				if (e.button === 2) {
					sX = true;
				}
				else {
					if (!e.shiftKey && noFit.cur && img.width >= 60
						&& img.width - (e.offsetX || e.layerX || 0) <= 30) {
						lastEvent.clientX = e.clientX;
						lastEvent.clientY = e.clientY;
						return;
					}

					e.preventDefault();

					win.focus();
					sX = e.clientX;
					sY = e.clientY;
				}

				if (!e.shiftKey) {
					doc.addEventListener("mousemove", onmove, false);
				}
			}

			if (!cfg.lpDelay) {
				return;
			}

			// for fine move and free zoom
			if (e.shiftKey) {
				cancelAction = true;

				if (e.button === 0) {
					fZoom = {
						counter: 0,
						left: parseInt(img.style.left, 10),
						top: parseInt(img.style.top, 10)
					};

					fZoom.X = e.clientX - fZoom.left;
					fZoom.Y = e.clientY - fZoom.top;

					if (!img.mask) {
						img.mask = doc.createElement("canvas");
						img.mask.className = "mask";
						img.mask.style.cssText = "display: block; position: fixed; left: 0; top: 0;";
						img.mctx = img.mask.getContext("2d");
					}

					doc.addEventListener("mousemove", draw_mask, false);

					var h = img.curdeg && Math.sin(img.curdeg) ? true : false,
						w = h ? img.offsetHeight : img.offsetWidth;
					h = h ? img.offsetWidth : img.offsetHeight;
					img.mask.width = Math.min(winW, w);
					img.mask.height = Math.min(winH, h);

					img.mask.style.left = img.style.left;
					img.mask.style.top = img.style.top;

					img.mctx.clearRect(0, 0, img.mask.width, img.mask.height);
					doc.body.appendChild(img.mask);

					w = win.getComputedStyle(img.mask).color;
					img.mctx.fillStyle = !w || w === "rgb(0, 0, 0)" ? "rgba(0,0,0,.4)" : w;
				}
			}

			// is dragSlideing
			if (progress) {
				if (stopScroll) {
					stopScroll();
					cancelAction = e.button === 0;
				}

				if (e.button === 0) {
					return;
				}
			}

			if (e.shiftKey || (e.button === 0 && !cfg.lpLeft)
				|| (e.button === 2 && !cfg.lpRight)) {
				return;
			}

			lastEvent.clientX = e.clientX;
			lastEvent.clientY = e.clientY;
			lastEvent.layerX = e.offsetX || e.layerX || 0;
			lastEvent.layerY = e.offsetY || e.layerY || 0;
			lastEvent.button = e.button;

			progress = setTimeout(longpress_handler, cfg.lpDelay);
		}, false);

		var longpress_handler = function() {
			cancelAction = true;
			progress = null;

			longpress_func(cfg[lastEvent.button === 2 ? "lpRight" : "lpLeft"]);
		},
		longpress_func = function(id) {
			switch (id) {
				case 1: // fit to width/height
					var x, y;

					if (img.mode === 2 || img.width === doc.body.clientWidth
						|| (doc.body.clientHeight > img.height && winW < doc.body.clientWidth)) {
						x = img.width;
						img.resize(3);
						x = lastEvent.layerX * img.width / x - winW / 2;
						y = 0;
					}
					else {
						y = img.height;
						img.resize(2);
						x = 0;
						y = lastEvent.layerY * img.height / y - winH / 2;
					}

					win.scrollTo(x, y);
				break;
			}
		};

		doc.addEventListener("mouseup", function(e) {
			if (e.button !== 0) {
				return;
			}

			var x, y;

			if (img.mode < 4) {
				doc.removeEventListener("mousemove", onmove, false);

				if (fZoom) {
					doc.removeEventListener("mousemove", draw_mask, false);
					doc.body.removeChild(img.mask);

					var
						w = Math.min(
							img.width,
							fZoom.w + Math.min(fZoom.x, 0)
								+ (fZoom.x + fZoom.w > img.mask.width ?
										img.mask.width - fZoom.x - fZoom.w : 0)
						),
						h = Math.min(
							img.height,
							fZoom.h + Math.min(fZoom.y, 0)
								+ (fZoom.y + fZoom.h > img.mask.height ?
										img.mask.height - fZoom.y - fZoom.h : 0)
						);

					x = Math.max(0, fZoom.x);
					y = Math.max(0, fZoom.y);

					fZoom = null;
					cancelAction = false;

					if (!w || !h || x >= img.width || y >= img.height) {
						return;
					}

					var nimgw, nimgh,
						scrollbars = (win.innerWidth - winW) || (win.innerHeight - winH);

					var fitW = winW < w * winH / h;

					fitW = e.ctrlKey ? !fitW : fitW;

					if (fitW) {
						nimgw = img.width * winW / w;
						nimgh = nimgw * img.height / img.width;
					}
					else {
						nimgh = img.height * winH / h;
						nimgw = nimgh * img.width / img.height;
					}

					// not needed in Opera 12, except for positioning
					if (nimgw > MAXSIZE) {
						nimgw = MAXSIZE;
						nimgh = img.height * MAXSIZE / img.width;
					}

					if (nimgh > MAXSIZE) {
						nimgw = img.width * MAXSIZE / img.height;
						nimgh = MAXSIZE;
					}

					var cx = (win.pageXOffset + x + w / 2) * nimgw /
										img.width - winW / 2 - scrollbars;
					var cy = (win.pageYOffset + y + h / 2) * nimgh /
										img.height - winH / 2 - scrollbars;

					img.resize(-1, nimgw + "px");

					win.scrollTo(cx, cy);
					return;
				}
				else if (dragSlide.length === 3) {
					x = dragSlide[0][0] - dragSlide[2][0];
					y = dragSlide[0][1] - dragSlide[2][1];

					// e.timeStamp
					if ((Date.now() - img.dragSlideTime) > 100) {
						cancelAction = false;
						dragSlide.length = 0;
						return;
					}

					if (x || y) {
						dragSlide.length = 2;
						dragSlide[0] = x;
						dragSlide[1] = y;
						startScroll();
						doc.addEventListener(browser["wheel"], stopScroll, false);
						return;
					}
				}
			}

			if (progress) {
				clearTimeout(progress);
				progress = null;
			}

			if (cancelAction) {
				cancelAction = false;
				return;
			}

			if (e.shiftKey || e.ctrlKey || e.altKey
				|| e.target !== img || lastEvent.button === null
				|| e.clientX !== lastEvent.clientX || e.clientY !== lastEvent.clientY) {
				return;
			}

			if (img.mode < 2 && noFit.real) {
				if (img.mode === -1) {
					img.resize(0);
				}
				else if (winW / winH < img.naturalWidth / img.naturalHeight
					|| winH === img.offsetHeight && winW > img.offsetWidth) {
					img.resize(2);
				}
				else {
					img.resize(3);
				}
			}
			else if (img.mode === 1 || noFit.cur) {
				if (img.naturalWidth === img.width
					&& img.naturalHeight === img.height) {
					return;
				}

				x = e.offsetX || e.layerX || 0;
				y = e.offsetY || e.layerY || 0;

				if (img.scale) {
					if (img.scale.h === -1) {
						x = img.width - x;
					}

					if (img.scale.v === -1) {
						y = img.height - y;
					}
				}

				x = x * img.naturalWidth / img.width - winW / 2;
				y = y * img.naturalHeight / img.height - winH / 2;

				img.resize(0);
				win.scrollTo(x, y);
			}
			else {
				img.resize(1);
			}
		}, false);

		doc.addEventListener("keydown", function(e) {
			if (stopScroll) {
				stopScroll();
			}

			if (shortcut.isModifier(e)) {
				return;
			}

			var key = shortcut.key(e);

			if (e.ctrlKey || (fZoom && key !== "Esc")) {
				return;
			}

			if (key === "+" || key === "-") {
				e.keypress = true;
				e.wheelDelta = key === "+" ? 1 : -1;
				e.clientX = winW / 2;
				e.clientY = winH / 2;
				e.offsetX = win.pageXOffset + img.offsetLeft + e.clientX;
				e.offsetY = win.pageYOffset + img.offsetTop + e.clientY;
				img.wheelZoom(e);
				return false;
			}

			var x, y, z;

			switch (key) {
				case "Esc":
					img.reset();
				break;

				case "PgUp":
					x = e.shiftKey ? -winW / 2 : 0;
					y = e.shiftKey ? 0 : -winH / 2;
				break;

				case "PgDn":
					x = e.shiftKey ? winW / 2 : 0;
					y = e.shiftKey ? 0 : winH / 2;
				break;

				case "End":
					x = e.shiftKey ? root.scrollWidth : win.pageXOffset;
					y = e.shiftKey ? win.pageYOffset : root.scrollHeight;
					z = true;
				break;

				case "Home":
					x = e.shiftKey ? 0 : win.pageXOffset;
					y = e.shiftKey ? win.pageYOffset : 0;
					z = true;
				break;

				default: x = true;
			}

			if (x !== true) {
				if (x !== void 0) {
					win[z ? "scrollTo" : "scrollBy"](x, y);
					e.preventDefault();
				}

				return;
			}

			switch (key) {
				case cfg.key_mOrig: img.resize(0); break;
				case cfg.key_mFit: img.resize(1); break;
				case cfg.key_mFitW: img.resize(2); break;
				case cfg.key_mFitH: img.resize(3); break;
				case cfg.key_cycle: img.cycle(); break;
				case cfg.key_rotate: img.rotate(!e.shiftKey, e.ctrlKey); break;
				case cfg.key_flip: transform.flip(img, e.shiftKey ? 0 : 1); break;

				default: x = true;
			}

			if (x !== true) {
				pdsp(e);
			}
		}, false);

		img.cycle = function(back) {
			var n_s = this.mode - (back ? 1 : -1);

			if (n_s === 1) {
				n_s = n_s - (back ? 1 : -1);
			}
			else if (n_s < 0) {
				n_s = 3;
			}
			else if (n_s > 3) {
				n_s = 0;
			}

			this.resize(n_s);
		};

		img.reset = function() {
			if (fZoom) {
				doc.removeEventListener("mousemove", draw_mask, false);
				doc.body.removeChild(img.mask);

				fZoom = null;
				cancelAction = false;
				return;
			}

			if (doc.readyState !== "complete") {
				return;
			}

			var filters = doc.body.querySelector("#menu li.filters > form");

			if (filters) {
				this.style.filter = this.style.webkitFilter = "";
				img.filters = {};
				filters.reset();
			}

			delete this.curdeg;
			delete this.scale;
			this.style[browser["transform"]] =
					this.style.width = this.style.height = "";
			this.resize(0);
		};

		img.rotate = function(deg, fine) {
			var rot;

			if (!this.curdeg) {
				this.curdeg = 0;
			}

			if (deg) {
				this.curdeg += (fine ? 10 : 90);
			}
			else {
				this.curdeg -= (fine ? 10 : 90);
			}

			win.status = this.curdeg + "°";

			rot = "rotate(" + this.curdeg + "deg)";

			if (this.scale) {
				rot += " scale(" + this.scale.h + ", " + this.scale.v + ")";
			}

			this.style[browser["transform"]] = rot;
			this.setPos();
		};

		img.mode = cfg.mode;
		img.calcFit();
		progress = [];

		if (cfg.minUpscale) {
			if (img.naturalWidth >= winW * cfg.minUpscale / 100) {
				progress[0] = 1;
			}

			if (img.naturalHeight >= winH * cfg.minUpscale / 100) {
				progress[1] = 1;
			}
		}

		if (img.mode === 4) {
			img.mode = img.naturalWidth / img.naturalHeight > winW / winH ? 3 : 2;
		}
		else if ((img.mode === 1 || (img.mode === 0 && noFit.real))
			&& progress.length) {
			img.mode = img.naturalWidth / img.naturalHeight > winW / winH ? 2 : 3;
		}

		if ((img.mode === 2 && img.naturalWidth < winW && !progress[0])
			|| (img.mode === 3 && img.naturalHeight < winH && !progress[1])) {
			img.mode = 0;
		}

		progress = null;
		img.resize(img.mode);
	};

	root = doc.documentElement;
	img = doc.createElement("img");

	img.addEventListener("error", function() {
		clearInterval(progress);
		doc.body.textContent = "";
		doc.body.appendChild(doc.createElement("h2")).textContent = _("LOAD_FAILED");
	}, false);

	img.src = URL;

	if (/^data:/.test(img.src)) {
		img.alt = _("VW_IMAGE") + " (data:)";

		if (!cfg.imgInfo) {
			doc.title = img.alt;
		}
	}
	else {
		img.alt = img.src.replace(/#.*/, "").match(/(?:[^\/]+)?$/)[0] || _("VW_IMAGE");

		try {
			// some Unicode characters caused problems
			img.alt = decodeURIComponent(img.alt);
		} catch (ex) {}
	}

	if (img.naturalWidth) {
		init_params = null;
		// Firefox and Safari won't position the image without setTimeout
		setTimeout(init, 0xf);
	}
	else {
		if (browser.opera || browser.firefox) {
			doc.body.textContent = "";
			progress = doc.body.appendChild(doc.createElement("h2"));
			progress.textContent = img.alt;
			progress.addEventListener("click", function() {
				doc.body.replaceChild(img, this);
			}, false);
		}

		init_params = {loop: 0, wait: 150, maxLoop: 900};
		progress = setInterval(init, init_params.wait);
	}
};


(function() {
	var is_image_document = function() {
		var d = document, img = d.images && d.images.length === 1 && d.images[0];
		return img && img.parentNode === d.body && img.src === window.location.href;
	};

	if (browser.opera || browser.firefox) {
		if (is_image_document()) {
			Port.listen(function(e) {
				Port.listen(null);
				init(window, document, e);
			});
			Port.send({"cmd": "hello"});
		}
		else {
			init = null;
		}
	}
	else {
		Port.listen(function(e) {
			Port.listen(null);

			if (!is_image_document()) {
				init = null;
				return;
			}

			init(window, document, e);
		});

		Port.send({"cmd": "hello"});
	}
})();
